#############################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0.1.2-1307  USA
#############################################################################

import gzip

from urllib2 import *
from cStringIO import StringIO


__all__ = ['HTTPGet']


class HTTPGet:
  """
  Wraps urllib2 for user-agent and gzip encoding.
    NOTE: gzip is disabled for .get() calls with 'saveas' parameter
    NOTE: authenticated proxy support remains untested
    NOTE: privoxy 3.0.6 forces uncompressed encoding (configurable?)
  """
  def __init__(self, proxy, agent, progresshook, gzip=True):
    self.agent = agent
    self.gzip = gzip
    self.progresshook = progresshook

    self._setopener(proxy)

  def get(self, url, saveas=None, referer=None):
    use_gzip = self.gzip
    if use_gzip and saveas: # Disable gzip if saveas is used
      use_gzip = False

    req = self._newreq(url,referer,use_gzip)
    #print req.headers
    o = self.opener.open(req)
    
    self.length = self._getlength(o.headers)
    self.data = ''
    self.read = 0
    self.file = None

    if saveas:
      self.file = open(saveas,'w')

    while 1:
      b = o.read(1024)
      if not b: break

      self._handledata(b)

      if self.progresshook:
        self.progresshook(self.read, self.length)

    if o.headers.get('content-encoding') == 'gzip':
      self.data = gzip.GzipFile(fileobj=StringIO(self.data)).read()
      if self.length == -1:
        self.length = len(self.data)

    o.close()

    if self.file:
      self.file.close()

  def reqlength(self, url, referer=None):
    # Store
    hook = self.progresshook
    self.progresshook = None
    
    req = self._newreq(url,referer,gzip=False)
    
    o = self.opener.open(req)
    length = self._getlength(o.headers)
    o.close()

    # Restore
    self.progresshook = hook

    return length

  # Non-public

  def _setopener(self, proxy):
    proxies = {} # Disable
    if proxy:
      if proxy != 'http_proxy':
        if proxy.lower() != 'none':
          proxies = {'http':proxy}
      else:
        proxies = None # Use http_proxy
    ph = ProxyHandler(proxies)
    self.opener = build_opener(ph)
    """
    auth = HTTPBasicAuthHandler()
    auth.add_password('realm','host','username','password')
    self.opener = build_opener(ph,auth)
    """

  def _newreq(self, url, referer, gzip):
    req = Request(url=url)
    if self.agent:
      req.add_header('user-agent',self.agent)
    if gzip:
      req.add_header('accept-encoding','gzip')
    if referer:
      req.add_header('referer',referer)
    return req

  def _handledata(self, data):
    self.read += len(data)
    if self.file:
      self.file.write(data)
    else:
      self.data += data

  def _getlength(self, headers):
    """Returns numeric content-length, sets None as -1"""
    length = headers.get('content-length')
    if not length:
      length = -1
    return long(length)
